@@ -1,4 +1,4 @@
-## Setup the default options
+# Setup the default options
 .PKNCAEnv <- new.env(parent=emptyenv())
 assign("options", NULL, envir=.PKNCAEnv)
 assign("summary", list(), envir=.PKNCAEnv)
@@ -7,27 +7,32 @@ assign("interval.cols", list(), envir=.PKNCAEnv)
 #' Add columns for calculations within PKNCA intervals
 #'
 #' @param name The column name as a character string
-#' @param FUN The function to run (as a character string) or \code{NA}
-#'   if the parameter is automatically calculated when calculating
-#'   another parameter.
+#' @param FUN The function to run (as a character string) or \code{NA} if the
+#'   parameter is automatically calculated when calculating another parameter.
 #' @param values Valid values for the column
-#' @param depends Character vector of columns that must be run before
-#'   this column.
-#' @param desc A human-readable description of the parameter (<=40
-#'   characters to comply with SDTM)
-#' @param formalsmap A named list mapping parameter names in the
-#'   function call to NCA parameter names.  See the details for information on use of \code{formalsmap}.
+#' @param depends Character vector of columns that must be run before this
+#'   column.
+#' @param desc A human-readable description of the parameter (<=40 characters to
+#'   comply with SDTM)
+#' @param sparse Is the calculation for sparse PK?
+#' @param unit_type The type of units to use for assigning and converting units.
+#' @param pretty_name The name of the parameter to use for printing in summary
+#'   tables with units.  (If an analysis does not include units, then the normal
+#'   name is used.)
+#' @param formalsmap A named list mapping parameter names in the function call
+#'   to NCA parameter names.  See the details for information on use of
+#'   \code{formalsmap}.
 #' @param datatype The type of data used for the calculation
-#' @return NULL (Calling this function has a side effect of
-#'   changing the available intervals for calculations)
+#' @return NULL (Calling this function has a side effect of changing the
+#'   available intervals for calculations)
 #'
 #' @details
-#' The \code{formalsmap} argument enables mapping some alternate formal
-#' argument names to parameters.  It is used to generalize functions
-#' that may use multiple similar arguments (such as the variants of mean
-#' residence time). The names of the list should correspond to function
-#' formal parameter names and the values should be one of the following:
-#' 
+#' The \code{formalsmap} argument enables mapping some alternate formal argument
+#' names to parameters.  It is used to generalize functions that may use
+#' multiple similar arguments (such as the variants of mean residence time). The
+#' names of the list should correspond to function formal parameter names and
+#' the values should be one of the following:
+#'
 #' \itemize{
 #'   \item{For the current interval:}
 #'   \describe{
@@ -61,28 +66,33 @@ assign("interval.cols", list(), envir=.PKNCAEnv)
 #' add.interval.col("cmax",
 #'                  FUN="pk.calc.cmax",
 #'                  values=c(FALSE, TRUE),
-#'                  desc="Maximum observed concentration",
-#'                  depends=c())
+#'                  unit_type="conc",
+#'                  pretty_name="Cmax",
+#'                  desc="Maximum observed concentration")
 #' add.interval.col("cmax.dn",
 #'                  FUN="pk.calc.dn",
 #'                  values=c(FALSE, TRUE),
+#'                  unit_type="conc_dosenorm",
+#'                  pretty_name="Cmax (dose-normalized)",
 #'                  desc="Maximum observed concentration, dose normalized",
 #'                  formalsmap=list(parameter="cmax"),
-#'                  depends=c("cmax"))
+#'                  depends="cmax")
 #' }
-#' @importFrom utils getAnywhere
 #' @family Interval specifications
 #' @export
 add.interval.col <- function(name,
                              FUN,
                              values=c(FALSE, TRUE),
-                             depends=c(),
+                             unit_type,
+                             pretty_name,
+                             depends=NULL,
                              desc="",
+                             sparse=FALSE,
                              formalsmap=list(),
                              datatype=c("interval",
                                "individual",
                                "population")) {
-  ## Check inputs
+  # Check inputs
   if (!is.character(name)) {
     stop("name must be a character string")
   } else if (length(name) != 1) {
@@ -93,6 +103,24 @@ add.interval.col <- function(name,
   } else if (!(is.character(FUN) | is.na(FUN))) {
     stop("FUN must be a character string or NA")
   }
+  checkmate::assert_logical(sparse, any.missing=FALSE, len=1)
+  unit_type <-
+    match.arg(
+      unit_type,
+      choices=c(
+        "unitless", "fraction", "%", "count",
+        "time", "inverse_time",
+        "amount",
+        "conc", "conc_dosenorm",
+        "volume",
+        "auc", "aumc",
+        "auc_dosenorm", "aumc_dosenorm",
+        "clearance", "renal_clearance"
+      )
+    )
+  stopifnot("pretty_name must be a scalar"=length(pretty_name) == 1)
+  stopifnot("pretty_name must be a character"=is.character(pretty_name))
+  stopifnot("pretty_name must not be an empty string"=nchar(pretty_name) > 0)
   datatype <- match.arg(datatype)
   if (!(datatype %in% "interval")) {
     stop("Only the 'interval' datatype is currently supported.")
@@ -113,7 +141,7 @@ add.interval.col <- function(name,
   } else if (!all(nchar(names(formalsmap)) > 0)) {
     stop("All formalsmap elements must be named")
   }
-  ## Ensure that the function exists
+  # Ensure that the function exists
   if (!is.na(FUN) &&
       length(utils::getAnywhere(FUN)$objs) == 0) {
     stop("The function named '", FUN, "' is not defined.  Please define the function before calling add.interval.col.")
@@ -131,7 +159,10 @@ add.interval.col <- function(name,
     list(
       FUN=FUN,
       values=values,
+      unit_type=unit_type,
+      pretty_name=pretty_name,
       desc=desc,
+      sparse=sparse,
       formalsmap=formalsmap,
       depends=depends,
       datatype=datatype
@@ -139,15 +170,19 @@ add.interval.col <- function(name,
   assign("interval.cols", current, envir=.PKNCAEnv)
 }
 
-## Add the start and end interval columns
+# Add the start and end interval columns
 add.interval.col("start",
   FUN = NA,
   values = as.numeric,
+  unit_type="time",
+  pretty_name="Interval Start",
   desc = "Starting time of the interval"
 )
 add.interval.col("end",
   FUN = NA,
   values = as.numeric,
+  unit_type="time",
+  pretty_name="Interval End",
   desc = "Ending time of the interval (potentially infinity)"
 )
 
@@ -156,30 +191,35 @@ add.interval.col("end",
 #' Columns are always to the right of columns that they depend on.
 sort.interval.cols <- function() {
   current <- get("interval.cols", envir=.PKNCAEnv)
-  ## Build a dependency tree
+  # Build a dependency tree
   myorder <- rep(NA, length(current))
   names(myorder) <- names(current)
   nextnum <- 1
-  while (any(is.na(myorder)))
-    for (nextorder in (1:length(myorder))[is.na(myorder)]) {
+  while (any(is.na(myorder))) {
+    for (nextorder in seq_along(myorder)[is.na(myorder)]) {
       if (length(current[[nextorder]]$depends) == 0) {
-        ## If it doesn't depend on anything then it can go next in
-        ## order.
+        # If it doesn't depend on anything then it can go next in order.
         myorder[nextorder] <- nextnum
         nextnum <- nextnum + 1
       } else {
-        ## If all of its dependencies already have values, then it can
-        ## be next.
+        # If all of its dependencies already have values, then it can be next.
         deps <- unique(unlist(current[[nextorder]]$depends))
-        if (!all(deps %in% names(myorder)))
-          stop("Invalid dependencies for interval column (please report this as a bug):", # nocov
-               names(myorder)[nextorder]) # nocov
+        missing_deps <- deps[!(deps %in% names(myorder))]
+        if (length(missing_deps) > 0) {
+          stop(
+            "Invalid dependencies for interval column (please report this as a bug): ",
+            names(myorder)[nextorder],
+            " The following dependencies are missing: ",
+            paste(missing_deps, collapse=", ")
+          )
+        }
         if (!any(is.na(myorder[deps]))) {
           myorder[nextorder] <- nextnum
           nextnum <- nextnum + 1
         }
       }
     }
+  }
   current <- current[names(sort(myorder))]
   assign("interval.cols", current, envir=.PKNCAEnv)
   invisible(myorder)
