@@ -229,9 +229,10 @@ pk.calc.aucint.inf.pred <- function(conc, time, start=NULL, end=NULL, time.dose,
 add.interval.col("aucint.last",
                  FUN="pk.calc.aucint.last",
                  values=c(FALSE, TRUE),
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUClast extrapolation)",
                  desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with zeros (matching AUClast)",
-                 formalsmap=list(conc="conc.group", time="time.group", time.dose=NULL),
-                 depends=c())
+                 formalsmap=list(conc="conc.group", time="time.group", time.dose=NULL))
 PKNCA.set.summary(
   name="aucint.last",
   description="geometric mean and geometric coefficient of variation",
@@ -242,9 +243,10 @@ PKNCA.set.summary(
 add.interval.col("aucint.last.dose",
                  FUN="pk.calc.aucint.last",
                  values=c(FALSE, TRUE),
-                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with zeros (matching AUClast)",
-                 formalsmap=list(conc="conc.group", time="time.group", time.dose="time.dose.group"),
-                 depends=c())
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUClast extrapolation, dose-aware)",
+                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with zeros (matching AUClast) with dose-aware interpolation/extrapolation of concentrations",
+                 formalsmap=list(conc="conc.group", time="time.group", time.dose="time.dose.group"))
 PKNCA.set.summary(
   name="aucint.last.dose",
   description="geometric mean and geometric coefficient of variation",
@@ -255,9 +257,10 @@ PKNCA.set.summary(
 add.interval.col("aucint.all",
                  FUN="pk.calc.aucint.all",
                  values=c(FALSE, TRUE),
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUCall extrapolation)",
                  desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with the triangle from Tlast to the next point and zero thereafter (matching AUCall)",
-                 formalsmap=list(conc="conc.group", time="time.group", time.dose=NULL),
-                 depends=c())
+                 formalsmap=list(conc="conc.group", time="time.group", time.dose=NULL))
 PKNCA.set.summary(
   name="aucint.all",
   description="geometric mean and geometric coefficient of variation",
@@ -268,9 +271,10 @@ PKNCA.set.summary(
 add.interval.col("aucint.all.dose",
                  FUN="pk.calc.aucint.all",
                  values=c(FALSE, TRUE),
-                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with the triangle from Tlast to the next point and zero thereafter (matching AUCall)",
-                 formalsmap=list(conc="conc.group", time="time.group", time.dose="time.dose.group"),
-                 depends=c())
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUCall extrapolation, dose-aware)",
+                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with the triangle from Tlast to the next point and zero thereafter (matching AUCall) with dose-aware interpolation/extrapolation of concentrations",
+                 formalsmap=list(conc="conc.group", time="time.group", time.dose="time.dose.group"))
 PKNCA.set.summary(
   name="aucint.all.dose",
   description="geometric mean and geometric coefficient of variation",
@@ -281,6 +285,8 @@ PKNCA.set.summary(
 add.interval.col("aucint.inf.obs",
                  FUN="pk.calc.aucint.inf.obs",
                  values=c(FALSE, TRUE),
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUCinf,obs extrapolation)",
                  desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with zeros (matching AUClast)",
                  formalsmap=list(conc="conc.group", time="time.group", time.dose=NULL),
                  depends=c("lambda.z", "clast.obs"))
@@ -294,7 +300,9 @@ PKNCA.set.summary(
 add.interval.col("aucint.inf.obs.dose",
                  FUN="pk.calc.aucint.inf.obs",
                  values=c(FALSE, TRUE),
-                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with zeros (matching AUClast)",
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUCinf,obs extrapolation, dose-aware)",
+                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with zeros (matching AUClast) with dose-aware interpolation/extrapolation of concentrations",
                  formalsmap=list(conc="conc.group", time="time.group", time.dose="time.dose.group"),
                  depends=c("lambda.z", "clast.obs"))
 PKNCA.set.summary(
@@ -307,6 +315,8 @@ PKNCA.set.summary(
 add.interval.col("aucint.inf.pred",
                  FUN="pk.calc.aucint.inf.pred",
                  values=c(FALSE, TRUE),
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUCinf,pred extrapolation)",
                  desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with the triangle from Tlast to the next point and zero thereafter (matching AUCall)",
                  formalsmap=list(conc="conc.group", time="time.group", time.dose=NULL),
                  depends=c("lambda.z", "clast.pred"))
@@ -320,7 +330,9 @@ PKNCA.set.summary(
 add.interval.col("aucint.inf.pred.dose",
                  FUN="pk.calc.aucint.inf.pred",
                  values=c(FALSE, TRUE),
-                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with the triangle from Tlast to the next point and zero thereafter (matching AUCall)",
+                 unit_type="auc",
+                 pretty_name="AUCint (based on AUCinf,pred extrapolation, dose-aware)",
+                 desc="The area under the concentration time curve in the interval extrapolating from Tlast to infinity with the triangle from Tlast to the next point and zero thereafter (matching AUCall) with dose-aware interpolation/extrapolation of concentrations",
                  formalsmap=list(conc="conc.group", time="time.group", time.dose="time.dose.group"),
                  depends=c("lambda.z", "clast.pred"))
 PKNCA.set.summary(
