@@ -1,40 +1,47 @@
 #' Create a PKNCAdata object.
-#' 
-#' \code{PKNCAdata} combines \code{PKNCAconc} and \code{PKNCAdose} and 
+#'
+#' \code{PKNCAdata} combines \code{PKNCAconc} and \code{PKNCAdose} and
 #' adds in the intervals for PK calculations.
-#' 
+#'
 #' @param data.conc Concentration data as a \code{PKNCAconc} object or a
 #'   data frame
-#' @param data.dose Dosing data as a \code{PKNCAdose} object (see 
+#' @param data.dose Dosing data as a \code{PKNCAdose} object (see
 #'   details)
+#' @param impute Methods for imputation.  \code{NA} for no imputation, a
+#'   comma-or space-separated list of names, or the name of a column in the
+#'   \code{intervals} data.frame.  See
+#'   \code{vignette("v08-data-imputation", package="PKNCA")} for more details.
 #' @param formula.conc Formula for making a \code{PKNCAconc} object with
-#'   \code{data.conc}.  This must be given if \code{data.conc} is a 
-#'   data.frame, and it must not be given if \code{data.conc} is a 
+#'   \code{data.conc}.  This must be given if \code{data.conc} is a
+#'   data.frame, and it must not be given if \code{data.conc} is a
 #'   \code{PKNCAconc} object.
 #' @param formula.dose Formula for making a \code{PKNCAdose} object with
-#'   \code{data.dose}.  This must be given if \code{data.dose} is a 
-#'   data.frame, and it must not be given if \code{data.dose} is a 
+#'   \code{data.dose}.  This must be given if \code{data.dose} is a
+#'   data.frame, and it must not be given if \code{data.dose} is a
 #'   \code{PKNCAdose} object.
 #' @param intervals A data frame with the AUC interval specifications as
 #'   defined in \code{\link{check.interval.specification}}.  If missing,
-#'   this will be automatically chosen by 
+#'   this will be automatically chosen by
 #'   \code{\link{choose.auc.intervals}}. (see details)
-#' @param options List of changes to the default 
+#' @param units A data.frame of unit assignments and conversions as created by
+#'   \code{\link{pknca_units_table}()}
+#' @param options List of changes to the default
 #'   \code{\link{PKNCA.options}} for calculations.
 #' @param ... arguments passed to \code{PKNCAdata.default}
-#' @return A PKNCAdata object with concentration, dose, interval, and 
-#'   calculation options stored (note that PKNCAdata objects can also 
+#' @return A PKNCAdata object with concentration, dose, interval, and
+#'   calculation options stored (note that PKNCAdata objects can also
 #'   have results after a NCA calculations are done to the data).
-#' @details If \code{data.dose} is not given or is \code{NA}, then the 
+#' @details If \code{data.dose} is not given or is \code{NA}, then the
 #'   \code{intervals} must be given.  At least one of \code{data.dose}
 #'   and \code{intervals} must be given.
 #' @family PKNCA objects
-#' @seealso \code{\link{choose.auc.intervals}}, \code{\link{pk.nca}}
+#' @seealso \code{\link{choose.auc.intervals}}, \code{\link{pk.nca}},
+#'   \code{\link{pknca_units_table}()}
 #' @export
 PKNCAdata <- function(data.conc, data.dose, ...)
   UseMethod("PKNCAdata", data.conc)
 
-## Ensure that arguments are reversible
+# Ensure that arguments are reversible
 #' @rdname PKNCAdata
 #' @export
 PKNCAdata.PKNCAconc <- function(data.conc, data.dose, ...)
@@ -43,7 +50,7 @@ PKNCAdata.PKNCAconc <- function(data.conc, data.dose, ...)
 #' @rdname PKNCAdata
 #' @export
 PKNCAdata.PKNCAdose <- function(data.conc, data.dose, ...) {
-  ## Swap the arguments
+  # Swap the arguments
   PKNCAdata.default(data.dose=data.conc, data.conc=data.dose, ...)
 }
 
@@ -51,33 +58,40 @@ PKNCAdata.PKNCAdose <- function(data.conc, data.dose, ...) {
 #' @export
 PKNCAdata.default <- function(data.conc, data.dose, ...,
                               formula.conc, formula.dose,
-                              intervals, options=list()) {
+                              impute = NA_character_,
+                              intervals, units, options=list()) {
   if (length(list(...))) {
     stop("Unknown argument provided to PKNCAdata.  All arguments other than `data.conc` and `data.dose` must be named.")
   }
   ret <- list()
-  ## Generate the conc element
+  # Generate the conc element
   if (inherits(data.conc, "PKNCAconc")) {
     if (!missing(formula.conc)) {
-      warning("data.conc was given as a PKNCAconc object.  Ignoring formula.conc")
+      rlang::warn(
+        message = "data.conc was given as a PKNCAconc object.  Ignoring formula.conc",
+        class = "pknca_dataconc_formulaconc"
+      )
     }
     ret$conc <- data.conc
   } else {
     ret$conc <- PKNCAconc(data.conc, formula=formula.conc)
   }
-  ## Generate the dose element
+  # Generate the dose element
   if (missing(data.dose)) {
     ret$dose <- NA
   } else if (identical(data.dose, NA)) {
     ret$dose <- NA
   } else if (inherits(data.dose, "PKNCAdose")) {
     if (!missing(formula.dose))
-      warning("data.dose was given as a PKNCAdose object.  Ignoring formula.dose")
+      rlang::warn(
+        message = "data.dose was given as a PKNCAdose object.  Ignoring formula.dose",
+        class = "pknca_dataconc_formuladose"
+      )
     ret$dose <- data.dose
   } else {
     ret$dose <- PKNCAdose(data.dose, formula.dose)
   }
-  ## Check the options
+  # Check the options
   if (!is.list(options)) {
     stop("options must be a list.")
   }
@@ -91,12 +105,12 @@ PKNCAdata.default <- function(data.conc, data.dose, ...,
     }
   }
   ret$options <- options
-  ## Check the AUC intervals
+  # Check the intervals
   if (missing(intervals) & identical(ret$dose, NA)) {
     stop("If data.dose is not given, intervals must be given")
   } else if (missing(intervals)) {
-    ## Generate the intervals for each grouping of concentration and
-    ## dosing.
+    # Generate the intervals for each grouping of concentration and
+    # dosing.
     if (identical(all.vars(parseFormula(ret$dose)$rhs), ".")) {
       stop("Dose times were not given, so intervals must be manually specified.")
     }
@@ -137,7 +151,10 @@ PKNCAdata.default <- function(data.conc, data.dose, ...,
           warning(warning_prefix, "No intervals generated likely due to limited concentration data")
         }
       } else {
-        warning(warning_prefix, "No intervals generated due to no concentration data")
+        rlang::warn(
+          message = paste(warning_prefix, "No intervals generated due to no concentration data"),
+          class = "pknca_no_intervals_generated"
+        )
       }
     }
     intervals <-
@@ -147,11 +164,37 @@ PKNCAdata.default <- function(data.conc, data.dose, ...,
       )
   }
   ret$intervals <- check.interval.specification(intervals)
-  ## Assign the class and give it all back to the user.
+  # Verify that either everything or nothing is using units
+  units_interval_start <- inherits(ret$intervals$start, "units")
+  units_interval_end <- inherits(ret$intervals$end, "units")
+
+  # Insert the unit conversion table
+  if (!missing(units)) {
+    stopifnot("`units` must be a data.frame"=is.data.frame(units))
+    stopifnot(
+      "`units` data.frame must have at least names 'PPTESTCD' and 'PPORRESU'"=
+        all(c("PPTESTCD", "PPORRESU") %in% names(units))
+    )
+    stopifnot("`units` must have at least one row"=nrow(units) > 0)
+    ret$units <- units
+  }
+
+  # Insert the imputation methods, if applicable
+  if (!identical(NA, impute)) {
+    checkmate::assert_character(impute, len = 1)
+    ret$impute <- impute
+  }
+
+  # Assign the class and give it all back to the user.
   class(ret) <- c("PKNCAdata", class(ret))
   ret
 }
 
+#' @rdname is_sparse_pk
+is_sparse_pk.PKNCAdata <- function(object) {
+  is_sparse_pk(object$conc)
+}
+
 #' Print a PKNCAdata object
 #' @param x The object to print
 #' @param ... Arguments passed on to \code{\link{print.PKNCAconc}} and
@@ -164,8 +207,14 @@ print.PKNCAdata <- function(x, ...) {
   } else {
     print.PKNCAdose(x$dose, ...)
   }
-  cat(sprintf("\nWith %d rows of AUC specifications.\n",
+  cat(sprintf("\nWith %d rows of interval specifications.\n",
               nrow(x$intervals)))
+  if (!is.null(x$units)) {
+    cat("With units\n")
+  }
+  if (!is.null(x$impute)) {
+    cat(sprintf("With imputation: %s\n", x$impute))
+  }
   if (length(x$options) == 0) {
     cat("No options are set differently than default.\n")
   } else {
@@ -174,18 +223,6 @@ print.PKNCAdata <- function(x, ...) {
   }
 }
 
-#' Extract all the original data from a PKNCAconc or PKNCAdose object
-#' @param object R object to extract the data from.
-#' @export
-getData.PKNCAdata <- function(object) {
-  object$data
-}
-
-#' @rdname getDataName
-getDataName.PKNCAdata <- function(object) {
-  "data"
-}
-
 #' Summarize a PKNCAdata object showing important details about the
 #' concentration, dosing, and interval information.
 #' @param object The PKNCAdata object to summarize.
