@@ -1,22 +1,22 @@
 #' Create a PKNCAdose object
-#' 
-#' @param data A data frame with time and the groups defined in 
+#'
+#' @param data A data frame with time and the groups defined in
 #'   \code{formula}.
-#' @param formula The formula defining the 
+#' @param formula The formula defining the
 #'   \code{dose.amount~time|groups} where \code{time} is the time of the
-#'   dosing and \code{dose.amount} is the amount administered at that 
+#'   dosing and \code{dose.amount} is the amount administered at that
 #'   time (see Details).
-#' @param route Define the route of administration.  The value may be 
-#'   either a column name from the \code{data} (checked first) or a 
-#'   character string of either \code{"extravascular"} or 
-#'   \code{"intravascular"} (checked second).  If given as a column 
-#'   name, then every value of the column must be either 
+#' @param route Define the route of administration.  The value may be
+#'   either a column name from the \code{data} (checked first) or a
+#'   character string of either \code{"extravascular"} or
+#'   \code{"intravascular"} (checked second).  If given as a column
+#'   name, then every value of the column must be either
 #'   \code{"extravascular"} or \code{"intravascular"}.
-#' @param rate,duration (optional) for \code{"intravascular"} dosing, 
+#' @param rate,duration (optional) for \code{"intravascular"} dosing,
 #'   the rate or duration of dosing.  If given as a character string, it
-#'   is the name of a column from the \code{data}, and if given as a 
-#'   number, it is the value for all doses.  Only one may be given, and 
-#'   if neither is given, then the dose is assumed to be a bolus 
+#'   is the name of a column from the \code{data}, and if given as a
+#'   number, it is the value for all doses.  Only one may be given, and
+#'   if neither is given, then the dose is assumed to be a bolus
 #'   (\code{duration=0}).  If \code{rate} is given, then the dose amount
 #'   must be given (the left hand side of the \code{formula}).
 #' @param time.nominal (optional) The name of the nominal time column
@@ -24,24 +24,24 @@
 #'   is not used during calculations; it is available to assist with
 #'   data summary and checking.
 #' @param exclude (optional) The name of a column with concentrations to
-#'   exclude from calculations and summarization.  If given, the column 
+#'   exclude from calculations and summarization.  If given, the column
 #'   should have values of \code{NA} or \code{""} for concentrations to
 #'   include and non-empty text for concentrations to exclude.
 #' @param ... Ignored.
 #' @return A PKNCAconc object that can be used for automated NCA.
-#' @details The \code{formula} for a \code{PKNCAdose} object can be 
+#' @details The \code{formula} for a \code{PKNCAdose} object can be
 #'   given three ways: one-sided (missing left side), one-sided (missing
-#'   right side), or two-sided.  Each of the three ways can be given 
-#'   with or without groups.  When given one-sided missing the left 
-#'   side, the left side can either be omitted or can be given as a 
-#'   period (\code{.}): \code{~time|treatment+subject} and 
+#'   right side), or two-sided.  Each of the three ways can be given
+#'   with or without groups.  When given one-sided missing the left
+#'   side, the left side can either be omitted or can be given as a
+#'   period (\code{.}): \code{~time|treatment+subject} and
 #'   \code{.~time|treatment+subject} are identical, and dose-related NCA
-#'   parameters will all be reported as not calculable (for example, 
+#'   parameters will all be reported as not calculable (for example,
 #'   clearance).  When given one-sided missing the right side, the right
-#'   side must be specified as a period (\code{.}): 
+#'   side must be specified as a period (\code{.}):
 #'   \code{dose~.|treatment+subject}, and only a single row may be given
-#'   per group.  When the right side is missing, PKNCA assumes that the 
-#'   same dose is given in every interval.  When given as a two-sided 
+#'   per group.  When the right side is missing, PKNCA assumes that the
+#'   same dose is given in every interval.  When given as a two-sided
 #'   formula
 #' @family PKNCA objects
 #' @export
@@ -61,20 +61,19 @@ PKNCAdose.tbl_df <- function(data, ...)
 #' @export
 PKNCAdose.data.frame <- function(data, formula, route, rate, duration,
                                  time.nominal, exclude, ...) {
-  ## The data must have... data
+  # The data must have... data
   if (nrow(data) == 0) {
     stop("data must have at least one row.")
   }
-  ## Check inputs
+  # Check inputs
   if (!missing(time.nominal)) {
     if (!(time.nominal %in% names(data))) {
       stop("time.nominal, if given, must be a column name in the input data.")
     }
   }
-  ## Verify that all the variables in the formula are columns in the
-  ## data.
+  # Verify that all the variables in the formula are columns in the data.
   parsedForm <- parseFormula(formula, require.two.sided=FALSE)
-  ## Check for variable existence and length
+  # Check for variable existence and length
   if (!(length(all.vars(parsedForm$lhs)) %in% c(0, 1)))
     stop("The left side of the formula must have zero or one variable")
   if (!(identical(parsedForm$lhs, NA) ||
@@ -89,7 +88,7 @@ PKNCAdose.data.frame <- function(data, formula, route, rate, duration,
   if (!all(all.vars(parsedForm$groups) %in% names(data))) {
     stop("All of the variables in the groups must be in the data")
   }
-  ## Values must be unique (one value per measurement)
+  # Values must be unique (one value per measurement)
   key.cols <- c(setdiff(all.vars(parsedForm$rhs), "."),
                 all.vars(parsedForm$groupFormula))
   if (any(mask.dup <- duplicated(data[,key.cols])))
@@ -110,9 +109,9 @@ PKNCAdose.data.frame <- function(data, formula, route, rate, duration,
     stop("Some but not all values are missing for the independent variable, please see the help for PKNCAdose for how to specify the formula and confirm that your data has dose times for all doses.")
   }
   if (missing(route)) {
-    ret <- setRoute.PKNCAdose(ret)
+    ret <- setRoute(ret)
   } else {
-    ret <- setRoute.PKNCAdose(ret, route)
+    ret <- setRoute(ret, route)
   }
   ret <- setDuration.PKNCAdose(ret, duration=duration,
                                rate=rate, dose=getDepVar.PKNCAdose(ret))
@@ -126,7 +125,7 @@ PKNCAdose.data.frame <- function(data, formula, route, rate, duration,
 }
 
 #' Set the dosing route
-#' 
+#'
 #' @param object A PKNCAdose object
 #' @param route A character string indicating one of the following:  the
 #'   column from the data which indicates the route of administration, a
@@ -162,7 +161,7 @@ setRoute.PKNCAdose <- function(object, route, ...) {
 }
 
 #' Set the duration of dosing or measurement
-#' 
+#'
 #' @param object An object to set a duration on
 #' @param duration The value to set for the duration or the name of the
 #'   column in the data to use for the duration.
@@ -176,17 +175,20 @@ setDuration <- function(object, ...)
 #' @rdname setDuration
 #' @export
 setDuration.PKNCAdose <- function(object, duration, rate, dose, ...) {
+  if (missing(dose)) {
+    dose <- as.character(parseFormula(object$formula)$lhs)
+  }
   if (missing(duration) & missing(rate)) {
     object <- setAttributeColumn(object=object, attr_name="duration", default_value=0,
                                  message_if_default="Assuming instant dosing (duration=0)")
-                                
+
   } else if (!missing(duration) & !missing(rate)) {
     stop("Both duration and rate cannot be given at the same time")
     # TODO: A consistency check could be done, but that would get into
     # requiring near-equal checks for floating point error.
   } else if (!missing(duration)) {
     object <- setAttributeColumn(object=object, attr_name="duration", col_or_value=duration)
-  } else if (!missing(rate) & !missing(dose)) {
+  } else if (!missing(rate) & !missing(dose) && !is.na(dose)) {
     tmprate <- getColumnValueOrNot(object$data, rate, "rate")
     tmpdose <- getColumnValueOrNot(object$data, dose, "dose")
     duration <- tmpdose$data[[tmpdose$name]]/tmprate$data[[tmprate$name]]
@@ -254,23 +256,16 @@ group_vars.PKNCAdose <- function(x) {
   group_vars.PKNCAconc(x)
 }
 
-#' @rdname getData.PKNCAconc
-#' @export
-#' @importFrom nlme getData
-getData.PKNCAdose <-  function(object)
-  object$data
-
 #' @rdname getDataName
 getDataName.PKNCAdose <- function(object)
   "data"
 
 #' @rdname print.PKNCAconc
 #' @export
-#' @importFrom utils head
 print.PKNCAdose <- function(x, n=6, summarize=FALSE, ...) {
   cat("Formula for dosing:\n ")
   print(stats::formula(x), showEnv=FALSE, ...)
-  if (!is.null(time_nom_data <- getAttributeColumn(x, attr_name="time.nominal", warn_missing=c()))) {
+  if (!is.null(time_nom_data <- getAttributeColumn(x, attr_name="time.nominal", warn_missing=character()))) {
     cat("Nominal time column is: ", names(time_nom_data), "\n", sep="")
   } else {
     cat("Nominal time column is not specified.\n")
@@ -305,3 +300,8 @@ print.PKNCAdose <- function(x, n=6, summarize=FALSE, ...) {
 #' @rdname print.PKNCAconc
 #' @export
 summary.PKNCAdose <- summary.PKNCAconc
+
+#' @export
+as.data.frame.PKNCAdose <- function(x, ...) {
+  x$data
+}
