@@ -1,6 +1,6 @@
 #' Parse a formula into its component parts.
 #'
-#' This function supports parsing 
+#' This function supports parsing
 #'
 #' This function extracts the left hand side (\code{lhs}), right hand
 #' side (\code{rhs}), groups (\code{groups} and as a formula,
@@ -10,7 +10,7 @@
 #' This function borrows heavily from the \code{parseGroupFormula}
 #' function in the doBy package.
 #'
-#' @param form the formula to extract into its parts 
+#' @param form the formula to extract into its parts
 #' @param require.groups is it an error not to have groups?
 #' @param require.two.sided is it an error to have a one-sided
 #' formula?
@@ -25,16 +25,15 @@
 #' }
 #' @examples
 #' parseFormula("a~b", require.groups=FALSE)
-#' ## parseFormula("a~b", require.groups=TRUE) # This is an error
+#' # parseFormula("a~b", require.groups=TRUE) # This is an error
 #' parseFormula("a~b|c")
 #' parseFormula("a~b|c")$groups
 #' @family Formula parsing
 #' @export
-#' @importFrom stats as.formula
 parseFormula <- function (form,
                           require.groups=FALSE,
                           require.two.sided=FALSE) {
-  ## If it is not a formula, make it a formula if possible
+  # If it is not a formula, make it a formula if possible
   if (!inherits(form, "formula")) {
     made.formula <- FALSE
     try({
@@ -44,8 +43,8 @@ parseFormula <- function (form,
     if (!made.formula)
       stop("form must be a formula object or coercable into one")
   }
-  ## Check how many sides the formula has and extract the left and
-  ## right sides
+  # Check how many sides the formula has and extract the left and
+  # right sides
   lhs <- findOperator(form, "~", "left")
   rhs <- findOperator(form, "~", "right")
   groups <- findOperator(rhs, "|", "right")
@@ -82,21 +81,6 @@ parseFormula <- function (form,
   ret
 }
 
-#' @importFrom stats formula
-print.parseFormula <- function(x, ...) {
-  if (identical(x$lhs, NA)) {
-    cat("A one-sided formula ")
-  } else {
-    cat("A two-sided formula ")
-  }
-  if (is.null(x$groups)) {
-    cat("without groups.\n  ")
-  } else {
-    cat("with groups.\n  ")
-  }
-  cat(deparse(stats::formula(x)), "\n")
-}
-
 #' Convert the parsed formula back into the original
 #'
 #' @param x The parsed formula object to revert to the original
@@ -108,7 +92,6 @@ print.parseFormula <- function(x, ...) {
 #' @return A formula (optionally with portions removed)
 #' @family Formula parsing
 #' @export
-#' @importFrom stats as.formula
 formula.parseFormula <- function(x, drop.groups=FALSE, drop.lhs=FALSE, ...) {
   if (identical(x$lhs, NA) | drop.lhs) {
     ret <- stats::as.formula(call("~", x$rhs))
@@ -138,22 +121,22 @@ findOperator <- function(x, op, side) {
   side <- match.arg(tolower(side),
                     choices=c("left", "right", "both"))
   if (inherits(x, "name")) {
-    ## This is a specific variable, we never found the operator going
-    ## down this branch of the tree.
+    # This is a specific variable, we never found the operator going
+    # down this branch of the tree.
     return(NULL)
   } else if (is.null(x)) {
     return(NULL)
   } else if (inherits(x, "call") |
              inherits(x, "formula") |
              inherits(x, "(")) {
-    ## This is all or part of a formula
+    # This is all or part of a formula
     op <- as.name(op)
     if (identical(x[[1]], op)) {
-      ## We found the operator
+      # We found the operator
       if (length(x) == 1) {
         stop("call or formula with length 1 found after finding the operator, unknown how to proceed") # nocov
       } else if (length(x) == 2) {
-        ## Unary operators have a right hand side only
+        # Unary operators have a right hand side only
         if (side == "left") {
           return(NA)
         } else if (side == "right") {
@@ -163,7 +146,7 @@ findOperator <- function(x, op, side) {
         }
         stop("Unknown side with a found unary operator") # nocov
       } else if (length(x) == 3) {
-        ## Binary operator
+        # Binary operator
         if (side == "left") {
           return(x[[2]])
         } else if (side == "right") {
@@ -174,10 +157,10 @@ findOperator <- function(x, op, side) {
         stop("Unknown side with a found binary operator") # nocov
       }
     } else {
-      ## Go down the left then right side of the tree
+      # Go down the left then right side of the tree
       if (length(x) == 1)
         stop("call or formula with length 1 found without finding the operator, unknown how to proceed")
-      ## First search the left side
+      # First search the left side
       ret <- findOperator(x[[2]], op, side)
       if ((identical(ret, NA) |
            is.null(ret)) &
@@ -185,7 +168,7 @@ findOperator <- function(x, op, side) {
         ret <- findOperator(x[[3]], op, side)
     }
   } else {
-    ## This should not happen-- find the class that the object is
+    # This should not happen-- find the class that the object is
     stop(sprintf("Cannot handle class %s",
          paste(class(x), sep=", ")))
   }
