@@ -6,7 +6,7 @@
 #' different magnitude by treatment (as a factor) and a random
 #' steady-state concentration and time to stead-state by subject (see
 #' \code{random.effects} argument).
-#' 
+#'
 #' @param \dots See \code{\link{pk.tss.data.prep}}
 #' @param tss.fraction The fraction of steady-state required for calling
 #'   steady-state
@@ -21,14 +21,14 @@
 #' @param verbose Describe models as they are run, show convergence of
 #'   the model (passed to the nlme function), and additional details
 #'   while running.
-#' 
+#'
 #' @return A scalar float for the first time when steady-state is
 #'   achieved or \code{NA} if it is not observed.
 #' @family Time to steady-state calculations
 #' @references
-#' Maganti L, Panebianco DL, Maes AL.  Evaluation of Methods for
-#' Estimating Time to Steady State with Examples from Phase 1 Studies.
-#' AAPS Journal 10(1):141-7. doi:10.1208/s12248-008-9014-y
+#' Maganti, L., Panebianco, D.L. & Maes, A.L. Evaluation of Methods for
+#' Estimating Time to Steady State with Examples from Phase 1 Studies. AAPS J
+#' 10, 141–147 (2008). https://doi.org/10.1208/s12248-008-9014-y
 #' @export
 pk.tss.monoexponential <- function(...,
                                    tss.fraction=0.9,
@@ -39,7 +39,7 @@ pk.tss.monoexponential <- function(...,
                                      "single"),
                                    check=TRUE,
                                    verbose=FALSE) {
-  ## Check inputs
+  # Check inputs
   modeldata <- pk.tss.data.prep(..., check=check)
   if (is.factor(tss.fraction) |
       !is.numeric(tss.fraction))
@@ -53,8 +53,8 @@ pk.tss.monoexponential <- function(...,
   } else if (tss.fraction < 0.8) {
     warning("tss.fraction is usually >= 0.8")
   }
-  ## Note that this will by default choose "population" if nothing is
-  ## requested.
+  # Note that this will by default choose "population" if nothing is
+  # requested.
   output <- match.arg(output, several.ok=TRUE)
   if (!("subject" %in% names(modeldata))) {
     if (any(c("population", "popind", "individual") %in% output)) {
@@ -63,8 +63,8 @@ pk.tss.monoexponential <- function(...,
       output <- setdiff(output, c("population", "popind", "individual"))
     }
   }
-  ## Set the tss.constant so that exp(tss.constant) == tss.fraction so
-  ## that the model below solves for the requested tss.
+  # Set the tss.constant so that exp(tss.constant) == tss.fraction so
+  # that the model below solves for the requested tss.
   modeldata$tss.constant <- log(1-tss.fraction)
   ret_population <-
     if (any(c("population", "popind") %in% output)) {
@@ -92,7 +92,7 @@ pk.tss.monoexponential <- function(...,
     } else if (!identical(NA, ret_individual)) {
       ret_individual
     } else {
-      stop("Error in selection of return values for pk.tss.monoexponential.  This is likely a bug.")
+      stop("Error in selection of return values for pk.tss.monoexponential.  This is likely a bug.") # nocov
     }
   ret
 }
@@ -102,27 +102,31 @@ pk.tss.monoexponential <- function(...,
 #'
 #' @param data The data used for the model
 #' @return a list with elements for each of the variables
-#' @importFrom stats median
 tss.monoexponential.generate.formula <- function(data) {
-  ## Setup the correct ctrough.ss by treatment or not.
+  # Setup the correct ctrough.ss by treatment or not.
   if ("treatment" %in% names(data)) {
-    ctrough.by <-
-      list("Ctrough.ss by treatment"=list(
-        formula=ctrough.ss~treatment-1,
-        ## Set the starting values for the ctrough.ss as the mean
-        ## concentration by treatment
-        start=dplyr::summarize_(
-          dplyr::group_by_(data, "treatment"),
-          .dots=list(conc.mean=~mean(conc)))$conc.mean))
+    ctrough_by <-
+      list(
+        "Ctrough.ss by treatment"=
+          list(
+            formula=ctrough.ss~treatment-1,
+            # Set the starting values for the ctrough.ss as the mean
+            # concentration by treatment
+            start=dplyr::summarize(
+              dplyr::grouped_df(data, vars = "treatment"),
+              dplyr::across(.cols=dplyr::all_of("conc"), .fns = mean)
+            )$conc
+          )
+      )
   } else {
-    ctrough.by <-
+    ctrough_by <-
       list("Single Ctrough.ss"=list(
         formula=ctrough.ss~1,
-        ## Set the starting values for the ctrough.ss as the mean
-        ## concentration.
+        # Set the starting values for the ctrough.ss as the mean
+        # concentration.
         start=mean(data$conc)))
   }
-  ## Try all combinations of Tss by treatment and single value
+  # Try all combinations of Tss by treatment and single value
   tss.by <- list(
     "Single value for Tss"=list(
       formula=tss~1,
@@ -132,15 +136,15 @@ tss.monoexponential.generate.formula <- function(data) {
       formula=tss~treatment,
       start=rep(stats::median(unique(data$time)),
         length(unique(data$treatment))))
-  ## Try combinations of random effects with Ctrough.ss and
-  ## Tss. (These are returned even if they aren't always used)
+  # Try combinations of random effects with Ctrough.ss and
+  # Tss. (These are returned even if they aren't always used)
   ranef.by <-
     list("Ctrough.ss and Tss"=list(formula=ctrough.ss+tss~1|subject),
          "Tss"=list(formula=tss~1|subject),
          "Ctrough.ss"=list(formula=ctrough.ss~1|subject))
-  ## Return the list of parameters to test
+  # Return the list of parameters to test
   list(
-    ctrough.by=ctrough.by,
+    ctrough.by=ctrough_by,
     tss.by=tss.by,
     ranef.by=ranef.by
   )
@@ -151,7 +155,7 @@ tss.monoexponential.generate.formula <- function(data) {
 #'
 #' This function is not intended to be called directly.  Please use
 #' \code{pk.tss.monoexponential}.
-#' 
+#'
 #' If no model converges, then the
 #' \code{tss.monoexponential.population} column will be set to NA. If
 #' the best model does not include a random effect for subject on Tss
@@ -169,8 +173,6 @@ tss.monoexponential.generate.formula <- function(data) {
 #' provided).  The columns will be named
 #' \code{tss.monoexponential.population} and/or
 #' \code{tss.monoexponential.popind}.
-#' @importFrom nlme fixef nlme ranef
-#' @importFrom stats AIC
 pk.tss.monoexponential.population <- function(data,
                                               output=c(
                                                 "population",
@@ -178,23 +180,23 @@ pk.tss.monoexponential.population <- function(data,
                                               verbose=FALSE) {
   output <- match.arg(output, several.ok=TRUE)
   test.formula <- tss.monoexponential.generate.formula(data)
-  ## Loop over creating all models
+  # Loop over creating all models
   models <- list()
   for (myctrough.ss in names(test.formula$ctrough.by)) {
     for (mytss in names(test.formula$tss.by)) {
       for (myranef in names(test.formula$ranef.by)) {
-        ## Describe the current model
+        # Describe the current model
         current.desc <-
           sprintf("Fixed effects of %s and %s; random effects of %s",
                   myctrough.ss, mytss, myranef)
         if (verbose)
           print(current.desc)
-        ## Be prepared for failure
+        # Be prepared for failure
         current.model <- NA
         current.aic <- NA
         current.model.summary <- "Did not converge"
         try({
-          ## Test the current model
+          # Test the current model
           current.model <-
             nlme::nlme(conc~ctrough.ss*(1-exp(tss.constant*time/tss)),
                        fixed=list(
@@ -206,14 +208,14 @@ pk.tss.monoexponential.population <- function(data,
                          test.formula$tss.by[[mytss]]$start),
                        data=data,
                        verbose=verbose)
-          ## If the model converges, get the summary and AIC out.
+          # If the model converges, get the summary and AIC out.
           if (!is.null(current.model)) {
             current.model.summary <- summary(current.model)
             current.aic <- stats::AIC(current.model)
           }
         }, silent=!verbose)
-        ## Put the current model (or model attempt) into the list of
-        ## models.
+        # Put the current model (or model attempt) into the list of
+        # models.
         models <-
           append(
             models,
@@ -229,9 +231,14 @@ pk.tss.monoexponential.population <- function(data,
       }
     }
   }
-  ## Find the best model of the set and return the output from that one.
+  # Find the best model of the set and return the output from that one.
   all.model.summary <- AIC.list(lapply(models, function(x) x$model))
-  rownames(all.model.summary) <- sapply(models, function(x) x$desc)
+  rownames(all.model.summary) <-
+    vapply(
+      X = models,
+      FUN = function(x) x$desc,
+      FUN.VALUE = ""
+    )
   if (verbose)
     print(all.model.summary)
   if (all(is.na(all.model.summary$AIC)) |
@@ -280,7 +287,7 @@ pk.tss.monoexponential.population <- function(data,
         )
     }
   }
-  ## Return the requested columns
+  # Return the requested columns
   ret[,intersect(c("subject", "treatment",
                    paste("tss.monoexponential", output, sep=".")),
                  names(ret)),
@@ -292,7 +299,7 @@ pk.tss.monoexponential.population <- function(data,
 #'
 #' This function is not intended to be called directly.  Please use
 #' \code{pk.tss.monoexponential}.
-#' 
+#'
 #' If no model converges, then the \code{tss.monoexponential.single}
 #' and/or \code{tss.monoexponential.individual} column will be set to
 #' NA.
@@ -308,42 +315,42 @@ pk.tss.monoexponential.population <- function(data,
 #' provided).  The columns will be named
 #' \code{tss.monoexponential.population} and/or
 #' \code{tss.monoexponential.popind}.
-#' @importFrom dplyr group_by_
-#' @importFrom nlme gnls
-#' @importFrom stats coef median
+#' @importFrom rlang .data
 pk.tss.monoexponential.individual <- function(data,
                                               output=c(
                                                 "individual",
                                                 "single"),
                                               verbose=FALSE) {
-  fit.tss <- function(d) {
-    tss <- NA_real_
-    try({
-      current.model <-
-        nlme::gnls(conc~ctrough.ss*(1-exp(tss.constant*time/tss)),
-                   params=list(
-                     ctrough.ss~1,
-                     tss~1),
-                   start=c(
-                     mean(d$conc),
-                     stats::median(unique(d$time))),
-                   data=d,
-                   verbose=verbose)
-      if (!is.null(current.model)) {
-        tss <- stats::coef(current.model)[["tss"]]
+  fit_tss <- function(d) {
+    current_model <-
+      try({
+        nlme::gnls(
+          conc~ctrough.ss*(1-exp(tss.constant*time/tss)),
+          params=list(
+            ctrough.ss~1,
+            tss~1),
+          start=c(
+            mean(d$conc),
+            stats::median(unique(d$time))),
+          data=d,
+          verbose=verbose
+        )
+      }, silent=!verbose)
+    tss <-
+      if (inherits(current_model, "try-error")) {
+        NA_real_
+      } else if (is.null(current_model)) {
+        NA_real_
+      } else {
+        stats::coef(current_model)[["tss"]]
       }
-    }, silent=!verbose)
-    if (is.null(tss)) {
-      tss <- NA_real_
-    }
     tss
   }
   output <- match.arg(output, several.ok=TRUE)
-  ## Run by treatment or a single value for the full data frame
+  # Run by treatment or a single value for the full data frame
   data_maybe_grouped <-
     if ("treatment" %in% names(data)) {
-      data %>%
-        dplyr::group_by_("treatment")
+      dplyr::grouped_df(data, vars = "treatment")
     } else {
       data
     }
@@ -351,12 +358,11 @@ pk.tss.monoexponential.individual <- function(data,
     data_maybe_grouped %>%
     dplyr::summarize(
       tss.monoexponential.single=
-        fit.tss(
+        fit_tss(
           data.frame(
             time=.$time,
             tss.constant=.$tss.constant,
             conc=.$conc,
-            treatment=.$treatment,
             stringsAsFactors=FALSE
           )
         )
@@ -365,32 +371,28 @@ pk.tss.monoexponential.individual <- function(data,
       "individual" %in% output) {
     data_grouped <-
       if (all(c("treatment", "subject") %in% names(data))) {
-        data %>%
-          group_by_("treatment", "subject")
+        dplyr::grouped_df(data, vars = c("treatment", "subject"))
       } else if ("subject" %in% names(data)) {
-        data %>%
-          group_by_("subject")
+        dplyr::grouped_df(data, vars="subject")
       } else {
         stop("Subject must be specified to have subject-level fitting")
       }
-    ret.sub <-
-      data_grouped %>%
-      dplyr::summarize_(
-        .dots=list(
-          tss.monoexponential.individual=
-            ~fit.tss(
-              data.frame(
-                time=time,
-                tss.constant=tss.constant,
-                conc=conc,
-                stringsAsFactors=FALSE
-              )
+    ret_sub <-
+      dplyr::summarize(
+        data_grouped,
+        tss.monoexponential.individual=
+          fit_tss(
+            data.frame(
+              time=.data$time,
+              tss.constant=.data$tss.constant,
+              conc=.data$conc,
+              stringsAsFactors=FALSE
             )
-        )
+          )
       )
-    ret <- merge(ret, ret.sub, all=TRUE)
+    ret <- merge(ret, ret_sub, all=TRUE)
   }
-  ## Return the requested columns
+  # Return the requested columns
   as.data.frame(
     ret[,c(intersect(names(ret), c("subject", "treatment")),
            paste("tss.monoexponential", output, sep=".")),
