@@ -1,5 +1,3 @@
-context("PKNCA option setting")
-
 test_that("PKNCA.options", {
   # Missing/incorrect option names give an error indicating all the
   # options that are missing.
@@ -58,7 +56,6 @@ test_that("PKNCA.options", {
                list(adj.r.squared.factor=0.0001,
                     max.missing=0.5,
                     auc.method="lin up/log down",
-                    conc_above=NA_real_,
                     conc.na="drop",
                     conc.blq=list(
                       first="keep",
@@ -317,8 +314,6 @@ test_that("PKNCA.choose.option", {
                info="PKNCA.choose.option gives the default option when nothing else is given")
 })
 
-context("PKNCA summary setting")
-
 test_that("PKNCA.set.summary input checking", {
   # Get the current state to reset it at the end
   initial.summary.set <- PKNCA.set.summary()
