@@ -1,8 +1,5 @@
-context("AUC and AUMC")
-
 test_that("pk.calc.auxc", {
-  ## #####
-  ## Verify input checks
+  # Verify input checks
 
   expect_error(pk.calc.auxc(conc=1:2, time=0:1, interval=2:1, method="linear"),
                regexp="The AUC interval must be increasing",
@@ -17,13 +14,17 @@ test_that("pk.calc.auxc", {
   expect_equal(v1, NA,
                info="Starting before the beginning time returns NA (not an error)")
 
-  ## All concentrations are NA, return NA
-  expect_warning(v2 <- pk.calc.auxc(conc=c(NA, NA), time=2:3, interval=c(1, 3),
-                                    method="linear"),
-                 info="All concentrations NA gives a warning")
-  expect_equal(v2, NA,
-               info="All concentrations NA is NA")
-  ## All concentrations are 0, return 0
+  # All concentrations are NA, return NA
+  expect_warning(
+    v2 <- pk.calc.auxc(conc=c(NA, NA), time=2:3, interval=c(1, 3), method="linear"),
+    regexp = "All concentration data are missing"
+  )
+  expect_equal(
+    v2,
+    structure(NA_real_, exclude="No data for AUC calculation"),
+    info="All concentrations NA is NA"
+  )
+  # All concentrations are 0, return 0
   expect_equal(
     pk.calc.auxc(
       conc=c(0, 0), time=2:3, interval=c(2, 3),
@@ -32,35 +33,33 @@ test_that("pk.calc.auxc", {
     structure(0, exclude="DO NOT EXCLUDE"),
     info="All zeros is zero"
   )
-  ## Concentrations mix 0 and NA, return 0
+  # Concentrations mix 0 and NA, return 0
   expect_equal(
     pk.calc.auxc(
-      conc=c(NA, 0, NA), time=2:4, interval=c(1, 3),
+      conc=c(NA, 0, NA, 0, NA), time=2:6, interval=c(1, 3),
       method="linear"
     ),
     structure(0, exclude="DO NOT EXCLUDE"),
     info="Mixed zeros and NA is still zero."
   )
-  ## Invalid integration method
+  # Invalid integration method
   expect_error(pk.calc.auxc(conc=c(NA, 0, NA), time=2:4, interval=c(1, 3),
                             method="foo"),
                info="Invalid integration methods are caught.")
 })
 
-test_that("pk.calc.auc", {
-  nullfun <- function(x) x
-  ## Linear AUC when the conc at the end of the interval is above LOQ;
-  ## lambda.z is unused
+test_that("pk.calc.auc: Linear AUC when the conc at the end of the interval is above LOQ", {
+  # lambda.z is unused
   tests <- list(AUCinf=as.numeric(NA),
                 AUClast=1.5,
                 AUCall=1.5)
   for (t in names(tests)) {
-    ## Note: using this structure ensures that there will not be
-    ## excessive warnings during testing.
+    # Note: using this structure ensures that there will not be
+    # excessive warnings during testing.
     if (t == "AUCinf") {
       runner <- expect_warning
     } else {
-      runner <- nullfun
+      runner <- identity
     }
     runner(v1 <- pk.calc.auc(conc=c(0, 1, 1),
                              time=0:2,
@@ -70,21 +69,21 @@ test_that("pk.calc.auc", {
                              method="linear"))
     expect_equal(v1, tests[[t]], info=t)
   }
+})
 
-  
-  ## Linear AUC when the conc at the end of the interval is BLQ;
-  ## lambda.z is used to extrapolate to the end of the interval.
-  ## Since lambda.z is NA, the result is NA.
+test_that("pk.calc.auc: Linear AUC when the conc at the end of the interval is BLQ, lambda.z missing", {
+  # lambda.z is used to extrapolate to the end of the interval.
+  # Since lambda.z is NA, the result is NA.
   tests <- list(AUCinf=as.numeric(NA),
                 AUClast=0.5,
                 AUCall=1)
   for (t in names(tests)) {
-    ## Note: using this structure ensures that there will not be
-    ## excessive warnings during testing.
+    # Note: using this structure ensures that there will not be
+    # excessive warnings during testing.
     if (t == "AUCinf") {
       runner <- expect_warning
     } else {
-      runner <- nullfun
+      runner <- identity
     }
     runner(v1 <- pk.calc.auc(conc=c(0, 1, 0),
                              time=0:2,
@@ -94,18 +93,20 @@ test_that("pk.calc.auc", {
                              method="linear"))
     expect_equal(v1, tests[[t]], info=t)
   }
+})
 
-  ## The same when lambda.z is given
+test_that("pk.calc.auc: Linear AUC when the conc at the end of the interval is BLQ, lambda.z given", {
+  # The same when lambda.z is given
   tests <- list(AUCinf=1.5,
                 AUClast=0.5,
                 AUCall=1)
   for (t in names(tests)) {
-    ## Note: using this structure ensures that there will not be
-    ## excessive warnings during testing.
+    # Note: using this structure ensures that there will not be
+    # excessive warnings during testing.
     if (t == "AUCinf") {
       runner <- expect_warning
     } else {
-      runner <- nullfun
+      runner <- identity
     }
     runner(v1 <- pk.calc.auc(conc=c(0, 1, 0),
                              time=0:2,
@@ -115,18 +116,19 @@ test_that("pk.calc.auc", {
                              method="linear"))
     expect_equal(v1, tests[[t]], info=t)
   }
+})
 
-  ## And when there are multiple BLQ values at the end
+test_that("pk.calc.auc: Linear AUC when when there are multiple BLQ values at the end, lambda.z given", {
   tests <- list(AUCinf=1.5,
                 AUClast=0.5,
                 AUCall=1)
   for (t in names(tests)) {
-    ## Note: using this structure ensures that there will not be
-    ## excessive warnings during testing.
+    # Note: using this structure ensures that there will not be
+    # excessive warnings during testing.
     if (t == "AUCinf") {
       runner <- expect_warning
     } else {
-      runner <- nullfun
+      runner <- identity
     }
     runner(v1 <- pk.calc.auc(conc=c(0, 1, rep(0, 5)),
                              time=0:6,
@@ -136,9 +138,10 @@ test_that("pk.calc.auc", {
                              method="linear"))
     expect_equal(v1, tests[[t]], info=t)
   }
+})
 
-  ## Confirm that center BLQ points are dropped, kept, or imputed
-  ## correctly.  Do this with both "linear" and "lin up/log down"
+test_that("pk.calc.auc: Confirm that center BLQ points are dropped, kept, or imputed correctly", {
+  # Do this with both "linear" and "lin up/log down"
   tests <- list(
     "linear"=list(
       AUCinf=1+1+0.5+1.5+1.5+1,
@@ -148,14 +151,14 @@ test_that("pk.calc.auc", {
       AUCinf=1+1+0.5+1.5+1/log(2)+1,
       AUClast=1+1+0.5+1.5+1/log(2),
       AUCall=1+1+0.5+1.5+1/log(2)))
-  for (t in names(tests))
+  for (t in names(tests)) {
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
+      # Note: using this structure ensures that there will not be
+      # excessive warnings during testing.
       if (n == "AUCinf") {
         runner <- expect_warning
       } else {
-        runner <- nullfun
+        runner <- identity
       }
       runner(v1 <- pk.calc.auc(conc=c(0, 2, 0, 1, 2, 1),
                                time=0:5,
@@ -166,8 +169,11 @@ test_that("pk.calc.auc", {
                                method=t))
       expect_equal(v1, tests[[t]][[n]], info=paste(t, n))
     }
+  }
+})
 
-  ## AUCall looks different when there are BLQs at the end
+test_that("pk.calc.auc: Confirm BLQ in the middle or end are calculated correctly", {
+  # AUCall looks different when there are BLQs at the end
   tests <- list(
     "linear"=list(
       AUCinf=1+1+0.5+1.5+1.5+1,
@@ -177,14 +183,14 @@ test_that("pk.calc.auc", {
       AUCinf=1+1+0.5+1.5+1/log(2)+1,
       AUClast=1+1+0.5+1.5+1/log(2),
       AUCall=1+1+0.5+1.5+1/log(2)+0.5))
-  for (t in names(tests))
+  for (t in names(tests)) {
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
+      # Note: using this structure ensures that there will not be
+      # excessive warnings during testing.
       if (n == "AUCinf") {
         runner <- expect_warning
       } else {
-        runner <- nullfun
+        runner <- identity
       }
       runner(v1 <- pk.calc.auc(conc=c(0, 2, 0, 1, 2, 1, 0, 0),
                                time=0:7,
@@ -195,10 +201,11 @@ test_that("pk.calc.auc", {
                                method=t))
       expect_equal(v1, tests[[t]][[n]], info=paste(t, n))
     }
+  }
 
-  ## When BLQ in the middle and end are dropped, you get different
-  ## answers.  (Note that first BLQ dropping would cause errors due to
-  ## starting times differing, so not tested here.)
+  # When BLQ in the middle and end are dropped, you get different
+  # answers.  (Note that first BLQ dropping would cause errors due to
+  # starting times differing, so not tested here.)
   tests <- list(
     "linear"=list(
       AUCinf=1+3+1.5+1.5+1,
@@ -208,14 +215,14 @@ test_that("pk.calc.auc", {
       AUCinf=1+2/log(2)+1.5+1/log(2)+1,
       AUClast=1+2/log(2)+1.5+1/log(2),
       AUCall=1+2/log(2)+1.5+1/log(2)+0.5))
-  for (t in names(tests))
+  for (t in names(tests)) {
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
+      # Note: using this structure ensures that there will not be
+      # excessive warnings during testing.
       if (n == "AUCinf") {
         runner <- expect_warning
       } else {
-        runner <- nullfun
+        runner <- identity
       }
       runner(v1 <- pk.calc.auc(conc=c(0, 2, 0, 1, 2, 1, 0, 0),
                                time=0:7,
@@ -231,8 +238,10 @@ test_that("pk.calc.auc", {
                    tests[[t]][[n]],
                    info=paste(t, n))
     }
-  
-  ## When AUCinf is requested with NA for lambda.z, the result is NA.
+  }
+})
+
+test_that("pk.calc.auc: When AUCinf is requested with NA for lambda.z, the result is NA", {
   tests <- list(
     "linear"=list(
       AUCinf=as.numeric(NA),
@@ -242,14 +251,14 @@ test_that("pk.calc.auc", {
       AUCinf=as.numeric(NA),
       AUClast=1+2/log(2)+1.5+1/log(2),
       AUCall=1+2/log(2)+1.5+1/log(2)+0.5))
-  for (t in names(tests))
+  for (t in names(tests)) {
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
+      # Note: using this structure ensures that there will not be
+      # excessive warnings during testing.
       if (n == "AUCinf") {
         runner <- expect_warning
       } else {
-        runner <- nullfun
+        runner <- identity
       }
       runner(v1 <- pk.calc.auc(conc=c(0, 2, 0, 1, 2, 1, 0, 0),
                                time=0:7,
@@ -265,9 +274,14 @@ test_that("pk.calc.auc", {
                    tests[[t]][[n]],
                    info=paste(t, n))
     }
-  ## Test NA at the beginning
+  }
+})
 
-  ## Test NA at the end
+# Test NA at the beginning is in "Confirm error with NA at the same time as
+# the beginning of the interval" below
+
+test_that("pk.calc.auc: Test NA at the end", {
+  # Test NA at the end
   tests <- list(
     "linear"=list(
       AUCinf=as.numeric(NA),
@@ -279,12 +293,12 @@ test_that("pk.calc.auc", {
       AUCall=1+2/log(2)+1.5+1/log(2)+1))
   for (t in names(tests))
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
+      # Note: using this structure ensures that there will not be
+      # excessive warnings during testing.
       if (n == "AUCinf") {
         runner <- expect_warning
       } else {
-        runner <- nullfun
+        runner <- identity
       }
       runner(v1 <- pk.calc.auc(conc=c(0, 2, 0, 1, 2, 1, NA, 0),
                                time=0:7,
@@ -300,8 +314,9 @@ test_that("pk.calc.auc", {
                    tests[[t]][[n]],
                    info=paste(t, n))
     }
-  
-  ## Test interpolation of times within the time interval
+})
+
+test_that("pk.calc.auc: interpolation of times within the time interval", {
   tests <- list(
     "linear"=list(
       AUCinf=1+3+1.5+1.5+1,
@@ -313,12 +328,12 @@ test_that("pk.calc.auc", {
       AUCall=1+2/log(2)+1.5+1/log(2)+0.5/log(2)))
   for (t in names(tests))
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
+      # Note: using this structure ensures that there will not be
+      # excessive warnings during testing.
       if (n == "AUCinf") {
         runner <- expect_warning
       } else {
-        runner <- nullfun
+        runner <- identity
       }
       runner(v1 <- pk.calc.auc(conc=c(0, 2, 0, 1, 2, 1, 0),
                                time=c(0:5, 7),
@@ -334,9 +349,9 @@ test_that("pk.calc.auc", {
                    tests[[t]][[n]],
                    info=paste(t, n))
     }
-  
-  ## Confirm warning with beginning of interval before the beginning of
-  ## time
+})
+
+test_that("pk.calc.auc: warning with beginning of interval before the beginning of time", {
   tests <- list(
     "linear"=list(
       AUCinf=NA,
@@ -346,35 +361,39 @@ test_that("pk.calc.auc", {
       AUCinf=NA,
       AUClast=NA,
       AUCall=NA))
-  for (t in names(tests))
+  for (t in names(tests)) {
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
+      # Note: using this structure ensures that there will not be
+      # excessive warnings during testing.
       expect_warning(
         v1 <- pk.calc.auc(conc=c(0, 2, 0, 1, 2, 1, 0),
                           time=c(0:5, 7),
-                          interval=c(-1, 6),
+                          interval=c(-1, Inf),
                           lambda.z=1,
                           auc.type=n,
                           conc.blq=list(
                             first="keep",
                             middle="drop",
                             last="keep"),
-                          method=t))
+                          method=t),
+        class="pknca_warn_auc_before_first"
+      )
       expect_equal(v1,
                    tests[[t]][[n]],
                    info=paste(t, n))
     }
-  
-  ## Requesting an AUC interval that starts after the last measurement
-  ## results in a warning, but will provide an answer (to be tested
-  ## elsewhere).
+  }
+})
+
+test_that("pk.calc.auc: warning with beginning of interval before the beginning of time", {
+  # Requesting an AUC interval that starts after the last measurement
+  # results in a warning, but will provide an answer (to be tested
+  # elsewhere).
   expect_warning(pk.calc.auc(conc=1:2, time=0:1, interval=2:3,
                              method="linear"),
                  regexp="AUC start time \\(2\\) is after the maximum observed time \\(1\\)")
 
-  ## Confirm error with NA at the same time as the beginning of the
-  ## interval
+  # Confirm error with NA at the same time as the beginning of the interval
   tests <- list(
     "linear"=list(
       AUCinf=NA,
@@ -386,47 +405,42 @@ test_that("pk.calc.auc", {
       AUCall=NA))
   for (t in names(tests))
     for (n in names(tests[[t]])) {
-      ## Note: using this structure ensures that there will not be
-      ## excessive warnings during testing.
-      if (n == "AUCinf") {
-        runner <- expect_warning
-      } else {
-        runner <- nullfun
-      }
       expect_warning(
         v1 <- pk.calc.auc(conc=c(NA, 2, 0, 1, 2, 1, 0),
                           time=c(0:5, 7),
-                          interval=c(0, 6),
+                          interval=c(0, Inf),
                           lambda.z=1,
                           auc.type=n,
                           conc.blq=list(
                             first="keep",
                             middle="drop",
                             last="keep"),
-                          method=t))
+                          method=t),
+        class = "pknca_warn_auc_before_first"
+      )
       expect_equal(v1,
                    tests[[t]][[n]],
                    info=paste(t, n))
     }
 
-  ## Confirm error with concentration and time not of equal lengths
+  # Confirm error with concentration and time not of equal lengths
   expect_error(pk.calc.auc(conc=c(1, 2, 3), time=c(1, 2)),
                regexp="Conc and time must be the same length")
 
-  ## Confirm error with time not monotonically increasing (less than)
+  # Confirm error with time not monotonically increasing (less than)
   expect_error(pk.calc.auc(conc=c(1, 2, 3), time=c(1, 2, 1)),
                regexp="Time must be monotonically increasing")
 
-  ## Confirm error with time not monotonically increasing (equal)
+  # Confirm error with time not monotonically increasing (equal)
   expect_error(pk.calc.auc(conc=c(1, 2, 3), time=c(1, 2, 2)),
                regexp="Time must be monotonically increasing")
 
-  ## Confirm that AUC method checking works
+  # Confirm that AUC method checking works
   expect_error(pk.calc.auc(conc=c(1, 2, 3), time=c(1, 2, 3), method="wrong"),
                regexp='should be one of',
                info="Method names are confirmed for pk.calc.auc")
 
-  ## Confirm that everything works even when check is FALSE
+  # Confirm that everything works even when check is FALSE
   expect_equal(
     pk.calc.auc.inf(conc=c(0, 1, 1, 0),
                     time=0:3,
@@ -462,8 +476,8 @@ test_that("pk.calc.auc.last", {
 })
 
 test_that("pk.calc.auc.inf", {
-  ## Just ensuring that it is a simple wrapper.  Computation testing
-  ## is done elsewhere.
+  # Just ensuring that it is a simple wrapper.  Computation testing
+  # is done elsewhere.
   expect_equal(
     pk.calc.auc.inf(conc=c(0, 1, 1, 0),
                     time=0:3,
@@ -486,8 +500,8 @@ test_that("pk.calc.auc.inf", {
 })
 
 test_that("pk.calc.auc.all", {
-  ## Just ensuring that it is a simple wrapper.  Computation testing
-  ## is done elsewhere.
+  # Just ensuring that it is a simple wrapper.  Computation testing
+  # is done elsewhere.
   expect_equal(
     pk.calc.auc.all(conc=c(0, 1, 1, 0),
                     time=0:3,
@@ -535,8 +549,8 @@ test_that("pk.calc.aumc", {
 
 
 test_that("pk.calc.aumc.inf", {
-  ## Just ensuring that it is a simple wrapper.  Computation testing
-  ## is done elsewhere.
+  # Just ensuring that it is a simple wrapper.  Computation testing
+  # is done elsewhere.
   expect_equal(
     pk.calc.aumc.inf(conc=c(0, 1, 1, 0),
                      time=0:3,
@@ -559,8 +573,8 @@ test_that("pk.calc.aumc.inf", {
 })
 
 test_that("pk.calc.aumc.all", {
-  ## Just ensuring that it is a simple wrapper.  Computation testing
-  ## is done elsewhere.
+  # Just ensuring that it is a simple wrapper.  Computation testing
+  # is done elsewhere.
   expect_equal(
     pk.calc.aumc.all(conc=c(0, 1, 1, 0),
                     time=0:3,
@@ -581,8 +595,8 @@ test_that("pk.calc.aumc.all", {
 })
 
 test_that("pk.calc.aumc.last", {
-  ## Just ensuring that it is a simple wrapper.  Computation testing
-  ## is done elsewhere.
+  # Just ensuring that it is a simple wrapper.  Computation testing
+  # is done elsewhere.
   expect_equal(
     pk.calc.aumc.last(conc=c(0, 1, 1, 0),
                     time=0:3,
@@ -658,3 +672,24 @@ test_that("pk.calc.aumc.inf.pred", {
       method="linear"),
     info="pk.calc.aumc.inf.pred is a simple wrapper around pk.calc.aumc.inf")
 })
+
+test_that("AUC with a single concentration measured should return NA (fix #176)", {
+  expect_equal(
+    pk.calc.auc.last(conc = 1, time = 0),
+    structure(
+      NA_real_,
+      exclude="AUC cannot be calculated with only one measured concentration"
+    )
+  )
+  # One concentration measured with others missing is sill excluded
+  expect_equal(
+    pk.calc.auxc(
+      conc=c(NA, 0, NA), time=2:4, interval=c(1, 3),
+      method="linear"
+    ),
+    structure(
+      NA_real_,
+      exclude="AUC cannot be calculated with only one measured concentration"
+    )
+  )
+})
