@@ -1,7 +1,5 @@
-context("Dose-normalized NCA functions")
-
 test_that("pk.calc.dn", {
-  ## Ensure correct calculation
+  # Ensure correct calculation
   expect_equal(pk.calc.dn(1, 5), 0.2)
   expect_equal(pk.calc.dn(NA, 5), NA_real_)
   expect_equal(pk.calc.dn(1, NA), NA_real_)
@@ -24,4 +22,4 @@ test_that("pk.calc.cmax", {
                  myres$result$PPORRES[myres$result$PPTESTCD %in% "auclast.dn"],
                myres$result$dose[myres$result$PPTESTCD %in% "auclast"],
                info="Dose normalization works when requested as a parameter in pk.nca")
-})
\ No newline at end of file
+})
