@@ -1,76 +1,82 @@
-context("Simple NCA functions")
-
 test_that("adj.r.squared", {
-  ## Ensure correct calculation
+  # Ensure correct calculation
   expect_equal(adj.r.squared(1, 5), 1)
   expect_equal(adj.r.squared(0.5, 5), 1-0.5*4/3)
 
-  ## Ensure that N must be an integer > 2
-  expect_equal(
-    expect_warning(
+  # Ensure that N must be an integer > 2
+  expect_warning(
+    expect_equal(
       adj.r.squared(1, 2),
-      regexp="n must be > 2"
+      structure(NA_real_, exclude="n must be > 2")
     ),
-    structure(NA_real_, exclude="n must be > 2")
+    regexp="n must be > 2"
   )
 })
 
 test_that("pk.calc.cmax", {
-  ## Confirm that all NAs give NA as an output
+  # Confirm that all NAs give NA as an output
   expect_warning(v1 <- pk.calc.cmax(NA))
   expect_equal(v1, NA)
   expect_warning(v2 <- pk.calc.cmax(c(NA, NA)))
   expect_equal(v2, NA)
 
-  ## Confirm that no NAs give the max value
+  # Confirm that no NAs give the max value
   expect_equal(pk.calc.cmax(c(1, 2)), 2)
   expect_equal(pk.calc.cmax(c(1, 2, 3)), 3)
 
-  ## Confirm that some NAs give the NA-removed maximum value
+  # Confirm that some NAs give the NA-removed maximum value
   expect_equal(pk.calc.cmax(c(1, NA, 3)), 3)
   expect_equal(pk.calc.cmax(c(NA, NA, 3)), 3)
   expect_equal(pk.calc.cmax(c(1, NA, NA)), 1)
   expect_equal(pk.calc.cmax(c(1, NA, 3, NA)), 3)
 
-  ## Confirm that no data gives NA with a warning
-  expect_warning(v3 <- pk.calc.cmax(c()))
+  # Confirm that no data gives NA with a warning
+  expect_warning(v3 <- pk.calc.cmax(numeric()))
   expect_equal(v3, NA)
 })
 
 test_that("pk.calc.cmin", {
-  ## Confirm that all NAs give NA as an output
+  # Confirm that all NAs give NA as an output
   expect_warning(v1 <- pk.calc.cmin(NA))
   expect_equal(v1, NA)
   expect_warning(v2 <- pk.calc.cmin(c(NA, NA)))
   expect_equal(v2, NA)
 
-  ## Confirm that no NAs give the min value
+  # Confirm that no NAs give the min value
   expect_equal(pk.calc.cmin(c(1, 2)), 1)
   expect_equal(pk.calc.cmin(c(1, 2, 3)), 1)
 
-  ## Confirm that some NAs give the NA-removed minimum value
+  # Confirm that some NAs give the NA-removed minimum value
   expect_equal(pk.calc.cmin(c(1, NA, 3)), 1)
   expect_equal(pk.calc.cmin(c(NA, NA, 3)), 3)
   expect_equal(pk.calc.cmin(c(1, NA, NA)), 1)
   expect_equal(pk.calc.cmin(c(1, NA, 3, NA)), 1)
 
-  ## Confirm that no data gives NA with a warning
-  expect_warning(v3 <- pk.calc.cmin(c()))
+  # Confirm that no data gives NA with a warning
+  expect_warning(v3 <- pk.calc.cmin(numeric()))
   expect_equal(v3, NA)
 })
 
 test_that("pk.calc.tmax", {
-  ## No data give a warning and NA
-  expect_warning(v1 <- pk.calc.tmax(c(), c()))
+  # No data give a warning and NA
+  expect_warning(expect_warning(
+    v1 <- pk.calc.tmax(numeric(), numeric()),
+    class = "pknca_conc_none"),
+    class = "pknca_time_none"
+  )
   expect_equal(v1, NA)
 
-  ## Either concentration or time is missing, give an error
-  expect_error(pk.calc.tmax(conc=c()),
-               regexp="time must be given")
-  expect_error(pk.calc.tmax(time=c()),
-               regexp="conc must be given")
+  # Either concentration or time is missing, give an error
+  expect_error(
+    pk.calc.tmax(conc=numeric()),
+    regexp="time must be given"
+  )
+  expect_error(
+    pk.calc.tmax(time=numeric()),
+    regexp="conc must be given"
+  )
 
-  ## It calculates tmax correctly based on the use.first option
+  # It calculates tmax correctly based on the use.first option
   expect_equal(pk.calc.tmax(c(1, 2), c(0, 1), first.tmax=TRUE),
                1)
   expect_equal(pk.calc.tmax(c(1, 2), c(0, 1), first.tmax=FALSE),
@@ -82,13 +88,17 @@ test_that("pk.calc.tmax", {
 })
 
 test_that("pk.calc.tlast", {
-  ## Either concentration or time is missing, give an error
-  expect_error(pk.calc.tlast(conc=c()),
-               regexp="time must be given")
-  expect_error(pk.calc.tlast(time=c()),
-               regexp="conc must be given")
+  # Either concentration or time is missing, give an error
+  expect_error(
+    pk.calc.tlast(conc=numeric()),
+    regexp="time must be given"
+  )
+  expect_error(
+    pk.calc.tlast(time=numeric()),
+    regexp="conc must be given"
+  )
 
-  ## It calculates tlast correctly
+  # It calculates tlast correctly
   expect_equal(pk.calc.tlast(c(1, 2), c(0, 1)),
                1)
   expect_equal(pk.calc.tlast(c(0, 0), c(0, 1)),
@@ -101,18 +111,33 @@ test_that("pk.calc.tlast", {
                1)
 })
 
+test_that("pk.calc.tfirst", {
+  expect_error(
+    pk.calc.tfirst(),
+    regexp="conc must be given"
+  )
+  expect_error(
+    pk.calc.tfirst(conc="A"),
+    regexp="time must be given"
+  )
+  expect_equal(
+    pk.calc.tfirst(conc=1, time=2, check=TRUE),
+    2
+  )
+})
+
 test_that("pk.calc.clast.obs", {
-  ## Ensure that it handles BLQ (0) values correctly
+  # Ensure that it handles BLQ (0) values correctly
   c1 <- c(0, 1, 2, 0)
   t1 <- c(0, 1, 2, 3)
   expect_equal(pk.calc.clast.obs(c1, t1), 2)
 
-  ## Ensure that it handles all ALQ values correctly
+  # Ensure that it handles all ALQ values correctly
   c1 <- c(0, 1, 2, 3)
   t1 <- c(0, 1, 2, 3)
   expect_equal(pk.calc.clast.obs(c1, t1), 3)
 
-  ## Ensure that it handles NA values correctly
+  # Ensure that it handles NA values correctly
   c1 <- c(0, 1, 2, NA)
   t1 <- c(0, 1, 2, 3)
   expect_equal(pk.calc.clast.obs(c1, t1), 2)
@@ -123,7 +148,10 @@ test_that("pk.calc.clast.obs", {
 
   c1 <- c(NA, NA, NA, NA)
   t1 <- c(0, 1, 2, 3)
-  expect_warning(v1 <- pk.calc.clast.obs(c1, t1))
+  expect_warning(
+    v1 <- pk.calc.clast.obs(c1, t1),
+    class = "pknca_conc_all_missing"
+  )
   expect_equal(v1, NA)
 
   c1 <- rep(0, 4)
@@ -132,15 +160,17 @@ test_that("pk.calc.clast.obs", {
 })
 
 test_that("pk.calc.thalf.eff", {
-  ## No input gives equivalent no output
-  expect_equal(pk.calc.thalf.eff(c()),
-               numeric())
+  # No input gives equivalent no output
+  expect_equal(
+    pk.calc.thalf.eff(numeric()),
+    numeric()
+  )
   
-  ## NA input gives equivalent NA output
+  # NA input gives equivalent NA output
   expect_equal(pk.calc.thalf.eff(NA),
                as.numeric(NA))
 
-  ## Numbers mixed with NA give appropriate output
+  # Numbers mixed with NA give appropriate output
   d1 <- c(0, 1, NA, 3)
   r1 <- log(2)*d1
   expect_equal(pk.calc.thalf.eff(d1),
@@ -148,22 +178,26 @@ test_that("pk.calc.thalf.eff", {
 })
 
 test_that("pk.calc.kel", {
-  ## No input gives equivalent no output
-  expect_equal(pk.calc.kel(c()),
-               numeric())
+  # No input gives equivalent no output
+  expect_equal(
+    pk.calc.kel(numeric()),
+    numeric()
+  )
   
-  ## NA input gives equivalent NA output
-  expect_equal(pk.calc.kel(NA),
-               as.numeric(NA))
+  # NA input gives equivalent NA output
+  expect_equal(
+    pk.calc.kel(NA),
+    as.numeric(NA)
+  )
 
-  ## Numbers mixed with NA give appropriate output
+  # Numbers mixed with NA give appropriate output
   d1 <- c(0, 1, NA, 3)
   r1 <- 1/d1
   expect_equal(pk.calc.kel(d1), r1)
 })
 
 test_that("pk.calc.cl", {
-  ## Ensure that dose and auc are required
+  # Ensure that dose and auc are required
   expect_error(pk.calc.cl(auc=NA),
                info="dose is required for clearance calculation")
   expect_error(pk.calc.cl(dose=NA),
@@ -212,8 +246,11 @@ test_that("pk.calc.aucpext", {
   expect_warning(v1 <- pk.calc.aucpext(2, 1),
                  regexp="aucpext is typically only calculated when aucinf is greater than auclast.")
   expect_equal(v1, -100)
-  expect_warning(v2 <- pk.calc.aucpext(auclast=0, aucinf=0),
-                 regexp="aucpext is typically only calculated when aucinf is greater than auclast.")
+  expect_warning(expect_warning(
+    v2 <- pk.calc.aucpext(auclast=0, aucinf=0),
+    class = "pknca_aucpext_aucinf_le_auclast"),
+    class = "pknca_aucpext_aucinf_auclast_positive"
+  )
   expect_equal(v2, NA_real_,
                info="aucinf<=0 gives NA_real_ (not infinity)")
   expect_equal(pk.calc.aucpext(NA, NA),
@@ -267,14 +304,14 @@ test_that("pk.calc.mrt.md", {
 })
 
 test_that("pk.calc.vz", {
-  ## Ensure that cl and lambda.z are required
+  # Ensure that cl and lambda.z are required
   expect_equal(pk.calc.vz(cl=NA, lambda.z=NA), NA_integer_)
   expect_error(pk.calc.vz(cl=NA),
                info="lambda.z required for Vz calculation")
   expect_error(pk.calc.vz(lambda.z=NA),
                info="CL required for Vz calculation")
 
-  ## Ensure that length of cl and lambda.z are either 1 or the same length
+  # Ensure that length of cl and lambda.z are either 1 or the same length
   expect_error(pk.calc.vz(cl=1:2, lambda.z=1:3),
                regexp="'cl' and 'lambda.z' must be the same length",
                info="CL and lambda.z must be the same length (CL shorter)")
@@ -282,8 +319,8 @@ test_that("pk.calc.vz", {
                regexp="'cl' and 'lambda.z' must be the same length",
                info="CL and lambda.z must be the same length (lambda.z shorter)")
   
-  ## Estimate a single Vz (with permutations to ensure the right math
-  ## is happening)
+  # Estimate a single Vz (with permutations to ensure the right math
+  # is happening)
   expect_equal(pk.calc.vz(cl=1, lambda.z=1), 1,
                info="vz math test 1")
   expect_equal(pk.calc.vz(cl=1, lambda.z=2), 0.5,
@@ -293,7 +330,7 @@ test_that("pk.calc.vz", {
   expect_equal(pk.calc.vz(cl=2, lambda.z=2), 1,
                info="vz math test 4")
 
-  ## Ensure that NA can go into either position or both
+  # Ensure that NA can go into either position or both
   expect_equal(pk.calc.vz(cl=NA, lambda.z=1), NA_integer_,
                info="Vz with missing (NA) cl")
   expect_equal(pk.calc.vz(cl=1, lambda.z=NA), NA_integer_,
@@ -301,7 +338,7 @@ test_that("pk.calc.vz", {
   expect_equal(pk.calc.vz(cl=NA, lambda.z=NA), NA_integer_,
                info="Vz with missing (NA) lambda.z and cl")
 
-  ## vectorized vz calculation works
+  # vectorized vz calculation works
   expect_equal(pk.calc.vz(cl=
                             c(1, 1, 1, 2, 2, 2, NA, NA, NA),
                           lambda.z=
@@ -401,4 +438,4 @@ test_that("pk.calc.ceoi", {
   expect_equal(pk.calc.ceoi(conc=0:5, time=0:5, duration.dose=NA),
                NA_real_,
                info="Ceoi returns NA if there is no dosing duration")
-})
\ No newline at end of file
+})
